#include "headers.h"

MODULE = Arisa::Channel  PACKAGE = Arisa::Network
PROTOTYPES: ENABLE

Arisa::Channel
channel_find(network, name = NULL)
	Arisa::Network network
	char *name
CODE:
	RETVAL.minor = network;
	if(irc_network_valid(network))
		RETVAL.major = irc_channel_find(network, name);
	else
		RETVAL.major = NULL;
OUTPUT:
	RETVAL

void
channels(network)
	Arisa::Network network
PREINIT:
	int i;
PPCODE:
	if(irc_network_valid(network)) {
		LOCK(network);
		for(i = 0; i < network->no_channels; ++i) {
			channel_t *c = network->channels[i];
			LOCK(c);
			if(c->deleted == 0) {
				XPUSHs(sv_2mortal(
					create_ptr2_object_p(
						"Arisa::Channel",c,network)));
			}
			UNLOCK(c);
		}
		UNLOCK(network);
	}

Arisa::Channel
channel_add(network, name = NULL, key = NULL)
	Arisa::Network network
	char *name
	char *key
CODE:
	RETVAL.major = NULL;
	RETVAL.minor = network;
	if(irc_network_valid(network) && name != NULL) {
		channel_t *c = alloc_channel();
		c->name = xstrdup(name);
		if(key != NULL)
			c->key = xstrdup(key);
		
		if(irc_channel_add(network,c) == 0)
			RETVAL.major = c;
		else
			free_channel(c);
	}
OUTPUT:
	RETVAL

	
#*******************************
MODULE = Arisa::Channel  PACKAGE = Arisa::Channel
#*******************************

SV*
delete(channel)
	Arisa::Channel channel
PREINIT:
	network_t *n;
	channel_t *c;
CODE:
	PTR2SPLIT(channel,channel_t,c,network_t,n);
	RETVAL = &PL_sv_no;
	if(irc_channel_valid(n,c)) {
		if(irc_channel_del(n,c) == 0)
			RETVAL = &PL_sv_yes;
	}
OUTPUT:
	RETVAL

SV*
name(channel, new_name = NULL)
	Arisa::Channel channel
	char *new_name
PREINIT:
	network_t *n;
	channel_t *c;
CODE:
	PTR2SPLIT(channel,channel_t,c,network_t,n);
	RETVAL = &PL_sv_no;
	if(irc_channel_valid(n,c)) {
		if(new_name != NULL) {
			if(irc_channel_rename(n,c,new_name) == 0)
				RETVAL = &PL_sv_yes;
		} else {
			LOCK(c);
			RETVAL = new_pv(c->name);
			UNLOCK(c);
		}
	}
OUTPUT:
	RETVAL

void
info(channel)
	Arisa::Channel channel
PREINIT:
	HV *hv = NULL;
	network_t *n;
	channel_t *c;
PPCODE:
	PTR2SPLIT(channel,channel_t,c,network_t,n);
	if(irc_channel_valid(n,c)) {
		hv = newHV();
		LOCK(c);
		hv_store(hv,"joined",6,
			c->joined != 0 ? newSVuv(c->joined) : &PL_sv_undef,0);
		hv_store(hv,"mode",4,new_pv(c->mode),0);
		hv_store(hv,"topic",5,new_pv(c->topic),0);
		hv_store(hv,"limit",5,newSViv(c->limit),0);
		hv_store(hv,"last_plist",10,
			c->last_plist ? newSVuv(c->last_plist):&PL_sv_undef,0);
		if(c->no_bans == 0) {
			hv_store(hv,"bans",4,&PL_sv_undef,0);
		} else {
			AV *av = newAV();
			int i;
			for(i = 0; i < c->no_bans; ++i)
				av_push(av,new_pv(c->bans[i]));
			hv_store(hv,"bans",4,newRV_noinc((SV*)av),0);
		}
		UNLOCK(c);
	}
	RET_REF(hv);

SV*
join(channel)
	Arisa::Channel channel
PREINIT:
	network_t *n;
	channel_t *c;
CODE:
	PTR2SPLIT(channel,channel_t,c,network_t,n);
	RETVAL = &PL_sv_no;
	if(irc_channel_valid(n,c)) {
		LOCK(n);
		if(n->state == NETWORK_CONNECTED) {
			LOCK(c);
			if(c->joined == 0) {
				char buffer[512];
				if(c->key == NULL)
					snprintf(buffer,sizeof(buffer),
						"JOIN %s",c->name);
				else
					snprintf(buffer,sizeof(buffer),
						"JOIN %s %s",c->name,c->key);
				UNLOCK(c);
				UNLOCK(n);
				if(irc_send_raw(n,buffer) == 0)
					RETVAL = &PL_sv_yes;
			} else {
				UNLOCK(c);
				UNLOCK(n);
			}
		} else {
			UNLOCK(n);
		}
	}
OUTPUT:
	RETVAL

SV*
part(channel, reason = NULL)
	Arisa::Channel channel
	char *reason
PREINIT:
	network_t *n;
	channel_t *c;
CODE:
	PTR2SPLIT(channel,channel_t,c,network_t,n);
	RETVAL = &PL_sv_no;
	if(irc_channel_valid(n,c)) {
		LOCK(c);
		if(c->joined != 0) {
			char buffer[512];
			if(reason != NULL)
				snprintf(buffer,sizeof(buffer),"PART %s :%s",
						c->name,reason);
			else
				snprintf(buffer,sizeof(buffer),"PART %s",
						c->name);
			UNLOCK(c);
			if(irc_send_raw(n,buffer) == 0)
				RETVAL = &PL_sv_yes;
		} else {
			UNLOCK(c);
		}
	}
OUTPUT:
	RETVAL

void
names(channel)
	Arisa::Channel channel
PREINIT:
	HV *hv = NULL;
	network_t *n;
	channel_t *c;
PPCODE:
	PTR2SPLIT(channel,channel_t,c,network_t,n);
	if(irc_channel_valid(n,c)) {
		hv = newHV();
		LOCK(n);
		if(n->info != NULL) {
			nickmode_t *nms = NULL;
			char buffer[4];
			int i,ret;
			
			LOCK(n->info);
			ret = nickhash_query_channel(n->info->nick_tracker,
					c,&nms);
			UNLOCK(n->info);
			
			for(i = 0; i < ret; ++i) {
				str_from_mode(buffer,sizeof(buffer),
						nms[i].mode);
				hv_store(hv,nms[i].nick,strlen(nms[i].nick),
						new_pv(buffer),0);
			}
			if(nms != NULL)
				xfree(nms);
		}
		UNLOCK(n);
	}
	RET_REF(hv);	
	
SV*
nolist(channel, ...)
	Arisa::Channel channel
PREINIT:
	network_t *n;
	channel_t *c;
CODE:
	PTR2SPLIT(channel,channel_t,c,network_t,n);
	RETVAL = &PL_sv_undef;
	if(irc_channel_valid(n,c)) {
		LOCK(c);
		if(items > 1)
			c->nolist = SvUV(ST(1));
		if(c->nolist != 0)
			RETVAL = newSVuv(c->nolist);
		UNLOCK(c);
	}
OUTPUT:
	RETVAL
	
SV*
send_msg(channel, msg = NULL)
	Arisa::Channel channel
	char *msg
PREINIT:
	network_t *n;
	channel_t *c;
CODE:
	PTR2SPLIT(channel,channel_t,c,network_t,n);
	RETVAL = &PL_sv_no;
	if(msg != NULL && irc_channel_valid(n,c)) {
		char buffer[256];
		LOCK(c);
		xstrncpy(buffer,c->name,sizeof(buffer));
		UNLOCK(c);
		if(irc_send_msg(n,buffer,msg) == 0)
			RETVAL = &PL_sv_yes;
	}
OUTPUT:
	RETVAL

SV*
send_notice(channel, msg = NULL)
	Arisa::Channel channel
	char *msg
PREINIT:
	network_t *n;
	channel_t *c;
CODE:
	PTR2SPLIT(channel,channel_t,c,network_t,n);
	RETVAL = &PL_sv_no;
	if(msg != NULL && irc_channel_valid(n,c)) {
		char buffer[256];
		LOCK(c);
		xstrncpy(buffer,c->name,sizeof(buffer));
		UNLOCK(c);
		if(irc_send_notice(n,buffer,msg) == 0)
			RETVAL = &PL_sv_yes;
	}
OUTPUT:
	RETVAL

void
settings(channel, ...)
	Arisa::Channel channel
PREINIT:
	HV *hv = NULL;
	network_t *n;
	channel_t *c;
PPCODE:
	PTR2SPLIT(channel,channel_t,c,network_t,n);
	if(irc_channel_valid(n,c)) {
		hv = newHV();
		perl_apply_settings(channel_apply_setting,c);
		perl_read_settings(channel_read_settings,c,hv);
	}
	RET_REF(hv);

void
taglines(channel, ...)
	Arisa::Channel channel
PREINIT:
	network_t *n;
	channel_t *c;
PPCODE:
	PTR2SPLIT(channel,channel_t,c,network_t,n);
	if(irc_channel_valid(n,c)) {
		LOCK(c);
		LOAD_RET_STRARR(c->taglines,c->no_taglines);
		UNLOCK(c);
	}
